@@ -1,4 +1,8 @@
-use jack_sys as j;
+#[cfg(feature = "dlopen")]
+use crate::LIB;
+use dlib::ffi_dispatch;
+#[cfg(not(feature = "dlopen"))]
+use jack_sys::*;
 use std::fmt;
 use std::fmt::Debug;
 use std::mem;
@@ -34,6 +38,9 @@ pub struct AsyncClient<N, P> {
     callback: Option<Box<CallbackContext<N, P>>>,
 }
 
+unsafe impl<N, P> Send for AsyncClient<N, P> {}
+unsafe impl<N, P> Sync for AsyncClient<N, P> {}
+
 impl<N, P> AsyncClient<N, P>
 where
     N: 'static + Send + Sync + NotificationHandler,
@@ -58,7 +65,12 @@ where
             });
             CallbackContext::register_callbacks(&mut callback_context)?;
             sleep_on_test();
-            let res = j::jack_activate(callback_context.client.raw());
+            let res = ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_activate,
+                callback_context.client.raw()
+            );
             for _ in 0..4 {
                 sleep_on_test();
             }
@@ -113,7 +125,7 @@ impl<N, P> AsyncClient<N, P> {
 
         // deactivate
         sleep_on_test();
-        if j::jack_deactivate(client) != 0 {
+        if ffi_dispatch!(feature = "dlopen", LIB, jack_deactivate, client) != 0 {
             return Err(Error::ClientDeactivationError);
         }
 
