@@ -1,4 +1,8 @@
-use jack_sys as j;
+#[cfg(feature = "dlopen")]
+use crate::LIB;
+use dlib::ffi_dispatch;
+#[cfg(not(feature = "dlopen"))]
+use jack_sys::*;
 use std::marker::PhantomData;
 use std::{mem, slice};
 
@@ -6,7 +10,7 @@ use crate::{Error, Frames, Port, PortFlags, PortSpec, ProcessScope};
 
 /// Contains 8bit raw midi information along with a timestamp relative to the
 /// process cycle.
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
 pub struct RawMidi<'a> {
     /// The amount of time passed, in frames, relative to the start of the
     /// process cycle
@@ -16,15 +20,6 @@ pub struct RawMidi<'a> {
     pub bytes: &'a [u8],
 }
 
-impl<'a> Default for RawMidi<'a> {
-    fn default() -> Self {
-        RawMidi {
-            time: 0,
-            bytes: &[],
-        }
-    }
-}
-
 /// `MidiIn` implements the `PortSpec` trait, which defines an endpoint for JACK. In this case, it
 /// defines midi input.
 #[derive(Copy, Clone, Debug, Default)]
@@ -37,7 +32,7 @@ pub struct MidiOut;
 
 unsafe impl PortSpec for MidiIn {
     fn jack_port_type(&self) -> &'static str {
-        j::RAW_MIDI_TYPE
+        jack_sys::RAW_MIDI_TYPE
     }
 
     fn jack_flags(&self) -> PortFlags {
@@ -91,8 +86,17 @@ impl<'a> MidiIter<'a> {
     }
 
     fn absolute_nth(&self, n: u32) -> Option<RawMidi<'a>> {
-        let mut ev = mem::MaybeUninit::<j::jack_midi_event_t>::uninit();
-        let res = unsafe { j::jack_midi_event_get(ev.as_mut_ptr(), self.buffer, n) };
+        let mut ev = mem::MaybeUninit::<jack_sys::jack_midi_event_t>::uninit();
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_midi_event_get,
+                ev.as_mut_ptr(),
+                self.buffer,
+                n
+            )
+        };
         if res != 0 {
             return None;
         }
@@ -106,7 +110,14 @@ impl<'a> MidiIter<'a> {
         if self.buffer.is_null() {
             0
         } else {
-            unsafe { j::jack_midi_get_event_count(self.buffer) as usize }
+            unsafe {
+                ffi_dispatch!(
+                    feature = "dlopen",
+                    LIB,
+                    jack_midi_get_event_count,
+                    self.buffer
+                ) as usize
+            }
         }
     }
 }
@@ -146,7 +157,7 @@ impl<'a> Iterator for MidiIter<'a> {
 
 unsafe impl PortSpec for MidiOut {
     fn jack_port_type(&self) -> &'static str {
-        j::RAW_MIDI_TYPE
+        jack_sys::RAW_MIDI_TYPE
     }
 
     fn jack_flags(&self) -> PortFlags {
@@ -165,7 +176,7 @@ impl Port<MidiOut> {
     pub fn writer<'a>(&'a mut self, ps: &'a ProcessScope) -> MidiWriter<'a> {
         assert_eq!(self.client_ptr(), ps.client_ptr());
         let buffer = unsafe { self.buffer(ps.n_frames()) };
-        unsafe { j::jack_midi_clear_buffer(buffer) };
+        unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_midi_clear_buffer, buffer) };
         MidiWriter {
             buffer,
             _phantom: PhantomData,
@@ -187,12 +198,22 @@ impl<'a> MidiWriter<'a> {
     /// realtime messages interspersed with other messagse (realtime messages are fine when they
     /// occur on their own, like other messages).
     pub fn write(&mut self, message: &RawMidi) -> Result<(), Error> {
-        let ev = j::jack_midi_event_t {
+        let ev = jack_sys::jack_midi_event_t {
             time: message.time,
             size: message.bytes.len(),
             buffer: message.bytes.as_ptr() as *mut u8,
         };
-        let res = unsafe { j::jack_midi_event_write(self.buffer, ev.time, ev.buffer, ev.size) };
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_midi_event_write,
+                self.buffer,
+                ev.time,
+                ev.buffer,
+                ev.size
+            )
+        };
         match res {
             0 => Ok(()),
             _ => Err(Error::NotEnoughSpace),
@@ -204,7 +225,14 @@ impl<'a> MidiWriter<'a> {
     /// If the return value is greater than 0, than the buffer is full.  Currently, the only way
     /// this can happen is if events are lost on port mixdown.
     pub fn lost_count(&self) -> usize {
-        let n = unsafe { j::jack_midi_get_lost_event_count(self.buffer) };
+        let n = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_midi_get_lost_event_count,
+                self.buffer
+            )
+        };
         n as usize
     }
 
@@ -213,7 +241,14 @@ impl<'a> MidiWriter<'a> {
     /// This function returns the current space available, taking into account events already stored
     /// in the port.
     pub fn max_event_size(&self) -> usize {
-        let n = unsafe { j::jack_midi_max_event_size(self.buffer) };
+        let n = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_midi_max_event_size,
+                self.buffer
+            )
+        };
         n as usize
     }
 }
